#!/usr/bin/env node

/***
 * 创建工程
 *  使用： feflow create <template-name> <project-name>
 *  注意：如果省略 project-name 则工程名默认为模版名
 * @author jiakaiguang
 * @time 2018-05-16
 */

const program = require('commander')
const path = require('path')
const exists = require('fs').existsSync
const inquirer = require('inquirer')
const ora = require('ora')
const chalk = require('chalk')
const home = require('user-home')
const rm = require('rimraf').sync
const fs = require('fs')
const shell = require('shelljs')

const logger = require('../lib/logger')
const {uncompress} = require('../lib/tar')
const {downloadFile} = require('../lib/stream')
const generate = require('../lib/generate')
const config = require('../config')

/**
 * Usage
 */
program
  .usage('<template-name> [project-name]')

/**
 * Help
 */
program.on('--help', () => {
  console.log('  Examples:')
  console.log()
  console.log(chalk.gray('  # create a new project with an official template'))
  console.log('  $ feflow create vue-pure my-project')
  console.log()
})

program.parse(process.argv)
if (program.args.length < 1) {
  return program.help()
}


let template = program.args[0] // 模版名称
const rawName = program.args[1] // 工程名称（raw）
const inPlace = !rawName || rawName === '.' // 是否在当前文件夹
const projectName = inPlace ? path.relative('../', process.cwd()) : rawName // 工程名称
const destPath = path.resolve(rawName || '.') // 工程存放路径
const templatePath = path.join(home, config.localTemplateDir, template.replace(/[\/:]/g, '-')) // 模版存放路径
const templateRoot = path.join(home, config.localTemplateDir)

// mkdir
if (!fs.existsSync(templateRoot)) {
  shell.mkdir('-p', templateRoot)
}

console.log()
process.on('exit', () => {
  console.log()
})

if (inPlace || exists(destPath)) {
  inquirer.prompt([{
    type: 'confirm',
    message: inPlace
      ? 'Generate project in current directory?'
      : 'Target directory exists. Continue?',
    name: 'ok'
  }]).then(answers => {
    if (answers.ok) {
      run()
    }
  }).catch(logger.fatal)
} else {
  run()
}


async function run() {
  try {
    const spinner = ora('downloading template...')
    spinner.start()

    // 如果模版在本地已存储，则先删除
    if (exists(templatePath)) {
      rm(templatePath)
    }

    // 下载模版
    if(!exists(templatePath)) {
      shell.mkdir('-p', templatePath)
    }
    const tarFile = `${templatePath}/${template}.tar.gz`
    await downloadFile(tarFile, `${config.remoteTempletePath}/${template}.tar.gz`)
    spinner.stop()

    // 解压
    uncompress(tarFile)

    // 删除压缩包
    rm(tarFile);

    // 生成
    await generate(templatePath, destPath)

    console.log()
    logger.success('√ generated project "%s".', projectName)
  } catch (err) {
    logger.fatal(err)
  }
}
