#!/usr/bin/env node
/***
 * 模版上传
 *  使用： feflow upload -s <source-template-path>
 *  注意：如果省略 -s 参数，则从当前路径进行验证
 * @author jiakaiguang
 * @time 2018-05-15
 */
const inquirer = require('inquirer')
const chalk = require('chalk')
const fs = require('fs')
const program = require('commander')
const path = require('path')
const home = require('user-home')
const shell = require('shelljs')

const templates = require('../template.json')
const logger = require('../lib/logger')
const {compress} = require('../lib/tar')
const {uploadFile} = require('../lib/stream')
const config = require('../config')

const TMP = 'tmp'


/**
 * Usage
 */
program
  .option('-s, --src', 'source template directory') // 源模版路径

/**
 * Help
 */
program.on('--help', () => {
  console.log('  Examples:')
  console.log()
  console.log(chalk.gray('  # upload a template'))
  console.log('  $ feflow upload')
  console.log()
})
program.parse(process.argv)


const src = program.src
let templatePath // 源模版路径
if (src) {
  templatePath = program.args[0]
} else {
  templatePath = process.cwd()
}

const localTemplatePath = path.join(home, config.localTemplateDir)

// 判断package.json 是否存在
const pkgfile = `${templatePath}/package.json`
if (!fs.existsSync(pkgfile)) {
  return logger.fatal('less file: package.json')
}
const pkg = require(pkgfile)


const questions = [
  {
    type: 'confirm',
    name: 'action',
    message: `Will you upgrade template ${pkg.name} to version ${pkg.version}?`,
    default: true
  }
]


inquirer.prompt(questions).then((o) => {
  if (!o.action) {
    return
  }

  // 当前版本与上传版本比较
  let curTemplate
  for (let template of templates) {
    if (template.name === pkg.name) {
      curTemplate = template
      break
    }
  }

  if (!curTemplate) {
    templates.push({
      name: pkg.name,
      description: pkg.description,
      version: pkg.version
    })
  } else {
    if (curTemplate.version === pkg.version) {
      return logger.fatal(`template version still "${pkg.version}", you should change it.`)
    }

    curTemplate.version = pkg.version
    curTemplate.description = pkg.description
  }

  try {

    // 创建压缩包
    const tarPath = path.resolve(localTemplatePath, TMP)
    const tarName = compress(templatePath, tarPath)

    // 上传
    uploadFile(`${tarPath}/${tarName}`, `${config.remoteTempletePath}/${tarName}`).then(() => {

      // 删除压缩包
      shell.rm('-rf', `${tarPath}/${tarName}`);

      // 更新template.json
      fs.writeFileSync(path.resolve(process.cwd(), 'template.json'), JSON.stringify(templates, null, 2), 'utf8')
    }).catch((err) => {
      return logger.fatal(err.message)
    })
  } catch (err) {
    return logger.fatal(err.message)
  }
})
